# xtsautorun/cli.py

import subprocess
import os
import time
import argparse
import os
import shutil
import re

#example command
#python3 xts-auto-run.py -r 3 -t /home/pedro/xTS/CTS/android-cts-12.1_r7-linux_x86-arm/android-cts/tools/cts-tradefed -c "run cts --abi arm64-v8a --shard-count 2 --subplan subplanCTS"

def get_connected_devices():
    result = subprocess.run(["adb", "devices"], capture_output=True, text=True)
    output_lines = result.stdout.strip().split('\n')
    
    # Extract device serial numbers
    devices = []
    for line in output_lines[1:]:  # Skip the header line
        device_info = line.split()
        if len(device_info) == 2 and device_info[1] == "device":
            devices.append(device_info[0])
    return devices

def reboot_and_wait(devices):
    for device in devices:
        subprocess.run(["adb", "-s", device, "reboot"])
    time.sleep(120)

    """for device in devices:
        while True:
            if device in get_connected_devices():
                print(f"Device {device} is online")
                break
            time.sleep(5)"""

def reboot_all():
    connected_devices = get_connected_devices()
    
    if not connected_devices:
        print("No connected Android devices found.")
    else:
        print("Connected devices:", connected_devices)
        reboot_and_wait(connected_devices)

    return connected_devices

def escape_special_characters(s):
    return s.replace('&', '\\&')

def connect_to_wifi_for_devices(wifi_name, wifi_password):
    devices = get_connected_devices()
    for device in devices:
        serial_number = device.split('\t')[0]
        escaped_wifi_password = escape_special_characters(wifi_password)
        enable_wifi_command = ["adb", "-s", serial_number, "shell", "svc", "wifi", "enable"]
        connect_wifi_command = ["adb", "-s", serial_number, "shell", "cmd", "-w", "wifi", "connect-network", wifi_name, "wpa2", escaped_wifi_password]
        try:
            subprocess.run(enable_wifi_command, check=True, capture_output=True, text=True)
            result = subprocess.run(connect_wifi_command, check=True, capture_output=True, text=True)
            if result.stdout == "Connection initiated \n":
                print(f"Connected to {wifi_name} on device with serial number: {serial_number}")
            else:
                print(f"Failed to connect to {wifi_name} on device with serial number: {serial_number}")
        except subprocess.CalledProcessError as e:
            print(f"Error executing command for device with serial number {serial_number}: {e.stderr}")

def organize_files(path):
    print(path)
    path_parts = path.split('/')
    path_parts = path_parts[:-2] 
    path_parts.append("results") 

    path = '/'.join(path_parts)

    files_to_move = [file for file in os.listdir(path) if not file.startswith("results")]

    if not files_to_move:
        print("No files to organize.")
        return

    first_file = files_to_move[0]
    folder_name = f"results_{first_file}"
    folder_path = os.path.join(path, folder_name)

    if not os.path.exists(folder_path):
        os.mkdir(folder_path)

    for file in files_to_move:
        src = os.path.join(path, file)
        dst = os.path.join(folder_path, file)
        shutil.move(src, dst)

    print(f"{len(files_to_move)} files moved to '{folder_name}' folder.")


def main():
    parser = argparse.ArgumentParser(description='Run CTS tests with retries')
    parser.add_argument('-r', required=True, help='Number of retries eg: 10')
    parser.add_argument('-t', required=True, help='Path to xTS tradefed eg: /home/pedro/xTS/CTS/android-cts-12.1_r7-linux_x86-arm/android-cts/tools/cts-tradefed')
    parser.add_argument('-c', required=True, help='xTS command to run eg: "run cts --abi arm64-v8a --shard-count 6 --subplan subplanCTS"')
    parser.add_argument('-w', required=True, help='WiFi network name, eg. ecarx-guest')
    parser.add_argument('-p', required=True, help='WiFi password, eg. "zdw5S3&v6M9B"')
    
    args = parser.parse_args()

    num_retries = int(args.r)
    CTS_trad_path = args.t
    command = args.c
    new_command = command.split(' ')
    command = str(new_command.insert(1, "commandAndExit"))
    result_string = ' '.join(new_command)
    wifi_name = args.w
    wifi_password = args.p

    print(f"Number of retries: {num_retries}")
    print(f"xTS tradefed path: {CTS_trad_path}")
    print(f"Command to run: {result_string}")
    print(f"Wifi Name: {wifi_name}")
    print(f"Wifi Password {wifi_password}")

    if not "retry "in result_string:
        print("no retry")
        organize_files(CTS_trad_path)
    else:
        print("retry")
        numbers = re.findall(r'--retry (\d+)', result_string)
        retry = numbers[0] if numbers else None
        print(retry)

    connect_to_wifi_for_devices(wifi_name, wifi_password)

    root_directory = "/"
    os.chdir(root_directory)

    command = f'.{CTS_trad_path} {result_string}'
    try:
        subprocess.run(command, shell=True, check=True)
    except subprocess.CalledProcessError as e:
        print(f'Command failed with error: {e}')
    except Exception as e:
        print(f'An error occurred: {e}')

    n_r = int(retry) + int(num_retries)
    for i in range(int(retry)+1, n_r):
        connected_devices_after_reboot = reboot_all()
        time.sleep(30)
        print(f'Retry number {i}')
        connect_to_wifi_for_devices(wifi_name, wifi_password)
        devs = get_connected_devices()
        command = f'.{CTS_trad_path} run commandAndExit retry --retry {i} --shard-count {len(devs)}'
        try:
            subprocess.run(command, shell=True, check=True)
        except subprocess.CalledProcessError as e:
            print(f'Command failed with error: {e}')
        except Exception as e:
            print(f'An error occurred: {e}')
        print(f'Retry {i} finished')
        connected_devices = reboot_all()
        time.sleep(2)

if __name__ == '__main__':
    main()